@@ -1,8 +1,8 @@
 /*******************************************************************************
    SecureSkat.cc, Secure Peer-to-Peer Implementation of the Card Game "Skat"
 
- Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2009
-               Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2009, 2016, 2017
+               Heiko Stamer <HeikoStamer@gmx.net>
 
    SecureSkat is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@ std::map<pid_t, std::string> nick_nick;         // map: PKI PID => nick name
 std::map<std::string, int> bad_nick;            // map: nick name => PKI attemps
 std::list<pid_t> nick_pids;                     // list: PIDs of PKI requests
 
-// FIXME: put all player data into a single structure
+// TODO: put all player data into a single structure
 std::map<std::string, std::string> nick_players;
 std::map<std::string, std::string> nick_package;
 std::map<std::string, int> nick_p7771, nick_p7772, nick_p7773, nick_p7774;
@@ -109,49 +109,37 @@ RETSIGTYPE sig_handler_usr1(int sig)
             {
                 // print success resp. error message
                 if (WEXITSTATUS(status) == 0)
-                    std::cerr << ">< " << _("Session") << " \"" << tnr << 
-                        "\" " << _("succeeded properly") << std::endl;
+                    std::cerr << ">< " << _("Session") << " \"" << tnr << "\" " << _("succeeded properly") << std::endl;
                 else
-                    std::cerr << ">< " << _("Session") << " \"" << tnr << 
-                        "\" " << _("failed. Error code") << ": WEXITSTATUS " << 
-                        WEXITSTATUS(status) << std::endl;
+                    std::cerr << ">< " << _("Session") << " \"" << tnr << "\" " << _("failed. Error code") << ": WEXITSTATUS " << WEXITSTATUS(status) << std::endl;
             }
             if (WIFSIGNALED(status))
             {
                 // print error message
-                std::cerr << ">< " << _("Session") << " \"" << tnr << "\" " << 
-                    _("failed. Error code") << ": WTERMSIG " <<
-                WTERMSIG(status) << std::endl;
+                std::cerr << ">< " << _("Session") << " \"" << tnr << "\" " << _("failed. Error code") << ": WTERMSIG " << WTERMSIG(status) << std::endl;
             }
             // remove associated data
             games_tnr2pid.erase(tnr);
             games_pid2tnr.erase(chld_pid);
         }
-        else if (std::find(rnkrpl_pid.begin(), rnkrpl_pid.end(), chld_pid) !=
-            rnkrpl_pid.end())
+        else if (std::find(rnkrpl_pid.begin(), rnkrpl_pid.end(), chld_pid) != rnkrpl_pid.end())
         {
             // print success message
-            std::cerr << ">< " << "RNK (pid = " << chld_pid << ") " << 
-                _("succeeded properly") << std::endl;
+            std::cerr << ">< " << "RNK (pid = " << chld_pid << ") " << _("succeeded properly") << std::endl;
             // remove associated data
             rnkrpl_pid.remove(chld_pid);
         }
-        else if (std::find(rnk_pids.begin(), rnk_pids.end(), chld_pid) !=
-            rnk_pids.end())
+        else if (std::find(rnk_pids.begin(), rnk_pids.end(), chld_pid) != rnk_pids.end())
         {
             if (WIFEXITED(status) && (WEXITSTATUS(status) != 0))
             {
                 // print error message
-                std::cerr << ">< " << "RNK (pid = " << chld_pid << ") " <<
-                    _("failed. Error code") << ": WEXITSTATUS " << 
-                    WEXITSTATUS(status) << std::endl;
+                std::cerr << ">< " << "RNK (pid = " << chld_pid << ") " << _("failed. Error code") << ": WEXITSTATUS " << WEXITSTATUS(status) << std::endl;
             }
             if (WIFSIGNALED(status))
             {
                 // print error message
-                std::cerr << ">< " << "RNK (pid = " << chld_pid << ") " << 
-                    _("failed. Error code") << ": WTERMSIG " << 
-                    WTERMSIG(status) << std::endl;
+                std::cerr << ">< " << "RNK (pid = " << chld_pid << ") " << _("failed. Error code") << ": WTERMSIG " << WTERMSIG(status) << std::endl;
             }
             // remove associated data
             rnk_pids.remove(chld_pid);
@@ -164,16 +152,12 @@ RETSIGTYPE sig_handler_usr1(int sig)
             if (WIFEXITED(status) && (WEXITSTATUS(status) != 0))
             {
                 // print error message
-                std::cerr << ">< " << "PKI " << chld_pid << "/" << 
-                    nick_nick[chld_pid] << " " << _("failed. Error code") << 
-                    ": WEXITSTATUS " << WEXITSTATUS(status) << std::endl;
+                std::cerr << ">< " << "PKI " << chld_pid << "/" << nick_nick[chld_pid] << " " << _("failed. Error code") << ": WEXITSTATUS " << WEXITSTATUS(status) << std::endl;
             }
             if (WIFSIGNALED(status))
             {
                 // print error message
-                std::cerr << ">< " << "PKI " << chld_pid << "/" << 
-                    nick_nick[chld_pid] << " " << _("failed. Error code") << 
-                    ": WTERMSIG " << WTERMSIG(status) << std::endl;
+                std::cerr << ">< " << "PKI " << chld_pid << "/" << nick_nick[chld_pid] << " " << _("failed. Error code") << ": WTERMSIG " << WTERMSIG(status) << std::endl;
             }
             // remove a bad nick (i.e. DoS attack on PKI) from the players list
             if (bad_nick.find(nick_nick[chld_pid]) == bad_nick.end())
@@ -203,8 +187,7 @@ RETSIGTYPE sig_handler_usr1(int sig)
         else
         {
 #ifndef NDEBUG
-            std::cerr << "sig_handler_usr1(): unknown child with PID " << 
-                chld_pid << std::endl;
+            std::cerr << "sig_handler_usr1(): unknown child with PID " << chld_pid << std::endl;
 #endif
         }
     } // end of while body
@@ -288,7 +271,7 @@ void pipe_irc(const std::string &irc_message)
                         std::endl << std::flush;
                 }
                 // Additionally, send the announcement to the main channel,
-				// because some IRC servers (e.g. freenode.net) may block
+		// because some IRC servers (e.g. freenode.net) may block
                 // private messages of unregistered users.
                 *irc << "PRIVMSG " << MAIN_CHANNEL << " :" << irc_parvec[1] << 
                     "~+~" << std::endl << std::flush;
@@ -348,124 +331,114 @@ void pipe_irc(const std::string &irc_message)
 
 void read_after_select(fd_set rfds, std::map<pid_t, int> &read_pipe, int what)
 {
-    std::vector<pid_t> del_pipe; // PIDs of pipes that should be closed
-    for (std::map<pid_t, int>::const_iterator pi = read_pipe.begin(); 
-        pi != read_pipe.end(); pi++)
+	std::vector<pid_t> del_pipe; // PIDs of pipes that should be closed
+	for (std::map<pid_t, int>::const_iterator pi = read_pipe.begin(); pi != read_pipe.end(); pi++)
 	{
-        int fd = pi->second;    // file descriptor of the pipe
-        size_t rbs = 65536;     // size of the read buffer
-        if ((fd >= 0) && FD_ISSET(fd, &rfds))
-        {
-            if (readbuf.find(fd) == readbuf.end())
+		int fd = pi->second;    // file descriptor of the pipe
+		size_t rbs = 65536;     // size of the read buffer
+		if ((fd >= 0) && FD_ISSET(fd, &rfds))
+		{
+			if (readbuf.find(fd) == readbuf.end())
 			{
-                // allocate a new read buffer for the pipe, if not exists
-                readbuf[fd] = new char[rbs];
-                // read buffer offset
-                readed[fd] = 0;
-            }
-			ssize_t num = read(fd, readbuf[fd] + readed[fd], rbs - readed[fd]);
-            readed[fd] += num;
-            if (num == 0)
-                del_pipe.push_back(pi->first); // close this pipe later
-            if (readed[fd] > 0)
-            {
-                std::vector<int> pos_delim; // positions of line delimiters
-                int cnt_delim = 0, cnt_pos = 0, pos = 0;
-                for (int i = 0; i < readed[fd]; i++)
-                    if (readbuf[fd][i] == '\n')
-                        cnt_delim++, pos_delim.push_back(i);
-                switch (what)
-                {
-                    case 1: // update of ranking data from RNK childs
-                        while (cnt_delim >= 2)
-                        {
-                            char *tmp = new char[rbs];
-                            std::memset(tmp, 0, rbs);
-                            std::memcpy(tmp, readbuf[fd] + cnt_pos,
-                                pos_delim[pos] - cnt_pos);
-                            --cnt_delim, cnt_pos = pos_delim[pos] + 1, pos++;
-                            std::string rnk1 = tmp;
+				// allocate a new read buffer for the pipe, if not exists
+				readbuf[fd] = new char[rbs];
+				// read buffer offset
+				readed[fd] = 0;
+			}
+			// read data from pipe
+			ssize_t num = 0;
+			if ((rbs - readed[fd]) > 0)
+			{
+				num = read(fd, readbuf[fd] + readed[fd], rbs - readed[fd]);
+				readed[fd] += num;
+				if (num == 0)
+					del_pipe.push_back(pi->first); // close this pipe later
+			}
+			// process data
+			if (readed[fd] > 0)
+			{
+				std::vector<int> pos_delim; // positions of line delimiters
+				int cnt_delim = 0, cnt_pos = 0, pos = 0;
+				for (int i = 0; i < readed[fd]; i++)
+					if (readbuf[fd][i] == '\n')
+						cnt_delim++, pos_delim.push_back(i);
+				char *tmp = new char[rbs]; // allocate temporary buffer of size rbs
+				switch (what)
+				{
+					case 1: // update of ranking data from RNK childs
+						while (cnt_delim >= 2)
+						{
 							std::memset(tmp, 0, rbs);
-							std::memcpy(tmp, readbuf[fd] + cnt_pos,
-								pos_delim[pos] - cnt_pos);
-							cnt_delim--, cnt_pos = pos_delim[pos] + 1, pos++;
+							std::memcpy(tmp, readbuf[fd] + cnt_pos, pos_delim[pos] - cnt_pos);
+							--cnt_delim, cnt_pos = pos_delim[pos] + 1, pos++;
+							std::string rnk1 = tmp;
+							std::memset(tmp, 0, rbs);
+							std::memcpy(tmp, readbuf[fd] + cnt_pos, pos_delim[pos] - cnt_pos);
+							--cnt_delim, cnt_pos = pos_delim[pos] + 1, pos++;
 							std::string rnk2 = tmp;
 							// do operation
 							rnk[rnk1] = rnk2;
-                            delete [] tmp;
 						}
 						break;
 					case 2: // IRC output from game childs
 						while (cnt_delim >= 1)
 						{
-							char *tmp = new char[rbs];
 							std::memset(tmp, 0, rbs);
-							std::memcpy(tmp, readbuf[fd] + cnt_pos,
-								pos_delim[pos] - cnt_pos);
+							std::memcpy(tmp, readbuf[fd] + cnt_pos, pos_delim[pos] - cnt_pos);
 							--cnt_delim, cnt_pos = pos_delim[pos] + 1, pos++;
 							std::string irc1 = tmp;
-
 //std::cerr << "to IRC: " << irc1 << std::endl;
-
+							// do operation
 							pipe_irc(irc1);
-                            delete [] tmp;
 						}
 						break;
 					case 3: // import from PKI childs
 						while (cnt_delim >= 2)
 						{
-							char *tmp = new char[rbs];
 							std::memset(tmp, 0, rbs);
-							std::memcpy(tmp, readbuf[fd] + cnt_pos,
-								pos_delim[pos] - cnt_pos);
+							std::memcpy(tmp, readbuf[fd] + cnt_pos, pos_delim[pos] - cnt_pos);
 							--cnt_delim, cnt_pos = pos_delim[pos] + 1, pos++;
 							std::string pki1 = tmp;
 							std::memset(tmp, 0, rbs);
-							std::memcpy(tmp, readbuf[fd] + cnt_pos,
-								pos_delim[pos] - cnt_pos);
-							cnt_delim--, cnt_pos = pos_delim[pos] + 1, pos++;
+							std::memcpy(tmp, readbuf[fd] + cnt_pos, pos_delim[pos] - cnt_pos);
+							--cnt_delim, cnt_pos = pos_delim[pos] + 1, pos++;
 							std::string pki2 = tmp;
 							// do operation
 							TMCG_PublicKey apkey;
 							if (!apkey.import(pki2))
 							{
-								std::cerr << _("TMCG: public key corrupted") <<
-                                    std::endl;
+								std::cerr << _("TMCG: public key corrupted") << std::endl;
 							}
 							else if (pki1 != apkey.keyid(5))
 							{
-								std::cerr << _("TMCG: wrong public key") <<
-                                    std::endl;
-                                std::cerr << pki1 << " vs. " << 
-                                    apkey.keyid(5) << std::endl;
+								std::cerr << _("TMCG: wrong public key") << std::endl;
+								std::cerr << pki1 << " vs. " << apkey.keyid(5) << std::endl;
 							}
 							else
 							{
 								std::cout << X << "PKI " << _("identified") <<
 									" \"" << pki1 << "\" " << "aka \"" << 
-                                    apkey.name << "\" <" << apkey.email << 
-                                    ">" << std::endl;
+									apkey.name << "\" <" << apkey.email << 
+									">" << std::endl;
 								nick_key[pki1] = apkey;
 							}
-                            delete [] tmp;
 						}
 						break;
 					default:
 						break;
 				} // end of switch
-				char *tmp = new char[rbs];
 				std::memset(tmp, 0, rbs);
 				readed[fd] -= cnt_pos;
 				std::memcpy(tmp, readbuf[fd] + cnt_pos, readed[fd]);
 				std::memcpy(readbuf[fd], tmp, readed[fd]);
-                delete [] tmp;
+				delete [] tmp;
 			}
 		}
 	}
-    // close dead pipes
+	// close dead pipes
 	for (size_t i = 0; i < del_pipe.size(); i++)
 	{
-        int fd = read_pipe[del_pipe[i]]; // file descriptor of the pipe
+		int fd = read_pipe[del_pipe[i]]; // file descriptor of the pipe
 		if (close(fd) < 0)
 			perror("read_after_select (close)");
 		delete [] readbuf[fd];
@@ -584,7 +557,7 @@ static void process_line(char *line)
             
 			// temporarily add the own public key
 			nick_key[pub.keyid(5)] = pub;
-            // parse the obtained RNK data
+			// parse the obtained RNK data
 			for (std::map<std::string, std::string>::const_iterator ri = 
 				rnk.begin(); ri != rnk.end(); ri++)
 			{
@@ -594,7 +567,7 @@ static void process_line(char *line)
 				std::string s = ri->second;
 				size_t ei;
 				
-                prt_counter++;
+				prt_counter++;
 				// header
 				if ((ei = s.find("#", 0)) != s.npos)
 				{
@@ -684,7 +657,7 @@ static void process_line(char *line)
 				else
 					continue;
 				if ((tk_header != "prt") ||
-                    // FIXME: sigid is ID8, but nick is ID5
+					// FIXME: sigid is ID8, but nick is ID5, we look only for a prefix
 					//(tk_nick[0] != pub.sigid(tk_sig1)) ||
 					//(tk_nick[1] != pub.sigid(tk_sig2)) ||
 					//(tk_nick[2] != pub.sigid(tk_sig3)) ||
@@ -701,7 +674,7 @@ static void process_line(char *line)
 					continue;
 				if (!nick_key[tk_nick[2]].verify(sig_data, tk_sig3))
 					continue;
-                prt_counter_valid++;
+				prt_counter_valid++;
 				std::list<std::string> gp_l;
 				std::string gp_s = "";
 				for (size_t j = 0; j < 3; j++)
@@ -747,8 +720,8 @@ static void process_line(char *line)
 					}
 				}
 			}
-            std::cout << prt_counter_valid << " out of " << prt_counter <<
-                " game protocols are valid." << std::endl;
+			std::cout << prt_counter_valid << " out of " << prt_counter <<
+				" game protocols are valid." << std::endl;
 			// Berechnen der Leistungspunkte (Erweitertes Seeger-System)
 			for (size_t j = 0; j < 3; j++)
 			{
@@ -811,7 +784,7 @@ static void process_line(char *line)
 					std::cout << "+----+ " << std::endl;
 				}
 			}
-            // remove the temporarily added key
+			// remove the temporarily added key
 			nick_key.erase(pub.keyid(5));
 		}
 		else if (cmd_argv[0] == "skat")
@@ -967,8 +940,7 @@ static void process_line(char *line)
 								int ret = ballot_child(tnr, b, true, in_pipe[0], out_pipe[1],
 									pub.keyid(5));
 #ifndef NDEBUG
-                                std::cerr << "ballot_child() = " << ret << 
-                                    std::endl;
+std::cerr << "ballot_child() = " << ret << std::endl;
 #endif
 								sleep(1);
 								if ((close(out_pipe[1]) < 0) || (close(in_pipe[0]) < 0))
@@ -1025,8 +997,7 @@ static void process_line(char *line)
 									int ret = ballot_child(tnr, -tables_r[tnr], false,
 										in_pipe[0], out_pipe[1], tables_o[tnr]);
 #ifndef NDEBUG
-                                    std::cerr << "ballot_child() = " << ret << 
-                                        std::endl;
+std::cerr << "ballot_child() = " << ret << std::endl;
 #endif
 									sleep(1);
 									if ((close(out_pipe[1]) < 0) || (close(in_pipe[0]) < 0))
@@ -1374,8 +1345,8 @@ void run_irc()
 				}
 			}
 			
-            // PKI (export public key on port 7771)
-            // -----------------------------------------------------------------
+			// PKI (export public key on port 7771)
+			// -----------------------------------------------------------------
 			if (FD_ISSET(pki7771_handle, &rfds))
 			{
 				struct sockaddr_in client_in;
@@ -1397,8 +1368,8 @@ void run_irc()
 				}
 			}
 			
-            // RNK (get rank entry on port 7774)
-            // -----------------------------------------------------------------
+			// RNK (get rank entry on port 7774)
+			// -----------------------------------------------------------------
 			if (FD_ISSET(rnk7774_handle, &rfds))
 			{
 				struct sockaddr_in client_in;
@@ -1434,14 +1405,12 @@ void run_irc()
 							signal(SIGTERM, SIG_DFL);
 							
 							/* BEGIN child code (ranking data) */
-							iosocketstream *client_ios =
-                                new iosocketstream(client_handle);
+							iosocketstream *client_ios = new iosocketstream(client_handle);
 							char *tmp = new char[100000L];
 							client_ios->getline(tmp, 100000L);
 							
 							if (rnk.find(tmp) != rnk.end())
-								*client_ios << rnk[tmp] << std::endl << 
-                                    std::flush;
+								*client_ios << rnk[tmp] << std::endl << std::flush;
 							else
 								*client_ios << std::endl << std::flush;
 							delete client_ios, delete [] tmp;
@@ -1459,16 +1428,16 @@ void run_irc()
 			}
 			
 #ifndef NOHUP
-            // read from stdin
-            // -----------------------------------------------------------------
+			// read from stdin
+			// -----------------------------------------------------------------
 			if (FD_ISSET(fileno(stdin), &rfds))
 			{
 				rl_callback_read_char();
 			}
 #endif
 			
-            // read from IRC connection
-            // -----------------------------------------------------------------
+			// read from IRC connection
+			// -----------------------------------------------------------------
 			if (FD_ISSET(irc_handle, &rfds))
 			{
 				ssize_t num = read(irc_handle, irc_readbuf + irc_readed, 
@@ -1477,9 +1446,7 @@ void run_irc()
 				
 				if (num == 0)
 				{
-					std::cerr <<
-                        _("IRC ERROR: connection with server collapsed") <<
-                        std::endl;
+					std::cerr << _("IRC ERROR: connection with server collapsed") << std::endl;
 					break;
 				}
 				
@@ -1493,22 +1460,20 @@ void run_irc()
 							cnt_delim++, pos_delim.push_back(i);
 						if (irc_readbuf[i] == '\015')
 							irc_readbuf[i] = '\n', cnt_delim++, 
-                                pos_delim.push_back(i);
+						pos_delim.push_back(i);
 					}
 					while (cnt_delim >= 1)
 					{
 						char tmp[65536];
 						memset(tmp, 0, sizeof(tmp));
-						memcpy(tmp, irc_readbuf + cnt_pos, 
-                               pos_delim[pos] - cnt_pos);
+						memcpy(tmp, irc_readbuf + cnt_pos, pos_delim[pos] - cnt_pos);
 						--cnt_delim, cnt_pos = pos_delim[pos] + 1, pos++;
 						std::string irc_reply = tmp;
 				
 				// parse NICK and HOST from IRC prefix
 				std::string pfx = irc_prefix(irc_reply);
 				std::string nick = "?", host = "?";
-				if ((pfx.find("!", 0) != pfx.npos) && 
-                    (pfx.find("@", 0) != pfx.npos))
+				if ((pfx.find("!", 0) != pfx.npos) && (pfx.find("@", 0) != pfx.npos))
 				{
 					nick = pfx.substr(0, pfx.find("!", 0));
 					host = pfx.substr(pfx.find("@", 0) + 1, 
@@ -1519,8 +1484,7 @@ void run_irc()
 				std::vector<std::string> irc_parvec;
 				if (irc_command_cmp(irc_reply, "PING"))
 				{
-					*irc << "PONG " << irc_params(irc_reply) << std::endl <<
-                        std::flush;
+					*irc << "PONG " << irc_params(irc_reply) << std::endl << std::flush;
 				} // USER success reply
 				else if (irc_command_cmp(irc_reply, "001"))
 				{
@@ -1536,9 +1500,7 @@ void run_irc()
 					irc_command_cmp(irc_reply, "466") ||
 					irc_command_cmp(irc_reply, "468"))
 				{
-					std::cerr << 
-                        _("IRC ERROR: not registered at IRC server") << 
-                        std::endl;
+					std::cerr << _("IRC ERROR: not registered at IRC server") << std::endl;
 					std::cerr << irc_reply << std::endl;
 					irc_quit = 1;
 					break;
@@ -1556,8 +1518,7 @@ void run_irc()
 							}
 							else if (nick.find(public_prefix, 0) == 0)
 							{
-								*irc << "WHO " << irc_parvec[0] << std::endl << 
-                                    std::flush;
+								*irc << "WHO " << irc_parvec[0] << std::endl << std::flush;
 							}
 							else
 							{
@@ -1905,8 +1866,7 @@ void run_irc()
 					if (irc_paramvec(irc_params(irc_reply), irc_parvec) >= 2)
 					{
 						// control messages
-						if ((irc_parvec[0].find(MAIN_CHANNEL_UNDERSCORE, 0) == 0) && 
-							(irc_parvec[0].length() > strlen(MAIN_CHANNEL_UNDERSCORE)) && 
+						if ((irc_parvec[0].find(MAIN_CHANNEL_UNDERSCORE, 0) == 0) && (irc_parvec[0].length() > strlen(MAIN_CHANNEL_UNDERSCORE)) && 
 							(nick.find(public_prefix, 0) == 0))
 						{
 							std::string tb = 
@@ -1918,8 +1878,7 @@ void run_irc()
 								if (tei != irc_parvec[1].npos)
 								{
 									std::string realmsg = irc_parvec[1].substr(0, tei);
-									std::string sig = irc_parvec[1].substr(tei + 3, 
-										irc_parvec[1].length() - realmsg.length() - 3);
+									std::string sig = irc_parvec[1].substr(tei + 3,	irc_parvec[1].length() - realmsg.length() - 3);
 									if (nick_key.find(nick) != nick_key.end())
 									{
 										if (nick_key[nick].verify(realmsg, sig))
@@ -1939,29 +1898,21 @@ void run_irc()
 							}
 						}
 						// announce and no channel messages
-						if (((irc_parvec[1].find("~+~") != irc_parvec[1].npos) && 
-								(irc_parvec[0] == MAIN_CHANNEL))
+						if (((irc_parvec[1].find("~+~") != irc_parvec[1].npos) && (irc_parvec[0] == MAIN_CHANNEL))
 							||
-							((irc_parvec[0] != MAIN_CHANNEL) &&
-								(nick.find(public_prefix, 0) == 0) &&
-								((irc_parvec[0] == pub.keyid(5)) &&
-									(nick != pub.keyid(5)))))
+							((irc_parvec[0] != MAIN_CHANNEL) && (nick.find(public_prefix, 0) == 0) &&
+								((irc_parvec[0] == pub.keyid(5)) && (nick != pub.keyid(5)))))
 						{
 							size_t tabei1 = irc_parvec[1].find("|", 0);
 							size_t tabei2 = irc_parvec[1].find("~", 0);
 							size_t tabei3 = irc_parvec[1].find("!", 0);
-							if ((tabei1 != irc_parvec[1].npos) &&
-								(tabei2 != irc_parvec[1].npos) &&
-								(tabei3 != irc_parvec[1].npos) && 
+							if ((tabei1 != irc_parvec[1].npos) && (tabei2 != irc_parvec[1].npos) && (tabei3 != irc_parvec[1].npos) && 
 								(tabei1 < tabei2) && (tabei2 < tabei3))
 							{
 								std::string tabmsg1 = irc_parvec[1].substr(0, tabei1);
-								std::string tabmsg2 = irc_parvec[1].substr(tabei1 + 1, 
-									tabei2 - tabei1 - 1);
-								std::string tabmsg3 = irc_parvec[1].substr(tabei2 + 1, 
-									tabei3 - tabei2 - 1);	
-								if ((std::find(tables.begin(), tables.end(), tabmsg1) 
-									== tables.end()) && (tabmsg2 != "0"))
+								std::string tabmsg2 = irc_parvec[1].substr(tabei1 + 1, tabei2 - tabei1 - 1);
+								std::string tabmsg3 = irc_parvec[1].substr(tabei2 + 1, tabei3 - tabei2 - 1);	
+								if ((std::find(tables.begin(), tables.end(), tabmsg1) == tables.end()) && (tabmsg2 != "0"))
 								{
 									// new table
 									tables.push_back(tabmsg1);
@@ -2006,9 +1957,7 @@ void run_irc()
 						else if (irc_stat && (irc_parvec[0] == MAIN_CHANNEL))
 						{
 							size_t tei = irc_parvec[1].find("~~~");
-							if ((nick.find(public_prefix, 0) == 0) &&
-								(nick_key.find(nick) != nick_key.end()) &&
-								(tei != irc_parvec[1].npos))
+							if ((nick.find(public_prefix, 0) == 0) && (nick_key.find(nick) != nick_key.end()) && (tei != irc_parvec[1].npos))
 							{
 								std::string realmsg = irc_parvec[1].substr(0, tei);
 								std::string sig = irc_parvec[1].substr(tei + 3, 
@@ -2024,9 +1973,7 @@ void run_irc()
 							else
 								std::cout << "<?" << nick << "?> " << irc_parvec[1] << std::endl;
 						} // other messages
-						else if (irc_stat &&
-							((irc_parvec[0] == pub.keyid(5)) &&
-							(nick != pub.keyid(5))))
+						else if (irc_stat && ((irc_parvec[0] == pub.keyid(5)) && (nick != pub.keyid(5))))
 						{
 							std::cout << ">?" << nick << "?< " << irc_parvec[1] << std::endl;
 						}
@@ -2083,28 +2030,32 @@ void run_irc()
 			if (first_command)
 			{
 				char ptmp[100];
-				snprintf(ptmp, sizeof(ptmp), "|%d~%d!%d#%d?%d/", 
-					pki7771_port, 0, rnk7773_port, rnk7774_port, 80);
+				snprintf(ptmp, sizeof(ptmp), "|%d~%d!%d#%d?%d/", pki7771_port, 0, rnk7773_port, rnk7774_port, 80);
 				std::string uname = pub.keyid(5);
-                // create a "unique" username based on the nickname
+				// create a "unique" username based on the nickname
 				if (uname.length() > 4)
 				{
-					std::string uname2 = "os";
+					std::string uname2 = "os"; // prefix
 					for (size_t ic = 4; ic < uname.length(); ic++)
 					{
 						if (islower(uname[ic]))
 							uname2 += uname[ic];
 						else if (isdigit(uname[ic]))
+						{
+							uname2 += "d"; // sign for decimal digit
 							uname2 += ('a' + (uname[ic] - 0x30));
+						}
 						else if (isupper(uname[ic]))
-							uname2 += ('a' + (uname[ic] - 0x40));
+						{
+							uname2 += "u"; // sign for upper case letter
+							uname2 += ('a' + (uname[ic] - 0x41));
+						}
 					}
 					uname = uname2;
 				}
 				else
 					uname = "unknown";
-				*irc << "USER " << uname << " 0 0 :" << PACKAGE_STRING << ptmp << 
-					std::endl << std::flush;
+				*irc << "USER " << uname << " 0 0 :" << PACKAGE_STRING << ptmp << std::endl << std::flush;
 				first_command = false;
 			}
 			else if (first_entry)
@@ -2119,8 +2070,7 @@ void run_irc()
 				// timer: autojoin to known tables each AUTOJOIN_TIMEOUT seconds
 				if (atj_counter >= AUTOJOIN_TIMEOUT)
 				{
-					for (std::list<std::string>::const_iterator ti = tables.begin(); 
-						ti != tables.end(); ti++)
+					for (std::list<std::string>::const_iterator ti = tables.begin(); ti != tables.end(); ti++)
 					{
 						// if not joined in a game, do AUTOJOIN (greedy behaviour)
 						if (games_tnr2pid.find(*ti) == games_tnr2pid.end())
@@ -2155,8 +2105,7 @@ void run_irc()
 				// timer: announce own tables every ANNOUNCE_TIMEOUT seconds
 				if (ann_counter >= ANNOUNCE_TIMEOUT)
 				{
-					for (std::map<pid_t, int>::const_iterator pi = games_ipipe.begin();
-						pi != games_ipipe.end(); ++pi)
+					for (std::map<pid_t, int>::const_iterator pi = games_ipipe.begin(); pi != games_ipipe.end(); ++pi)
 					{
 						opipestream *npipe = new opipestream(pi->second);
 						*npipe << "!ANNOUNCE" << std::endl << std::flush;
@@ -2169,8 +2118,7 @@ void run_irc()
 			}
 			
 			// send SIGQUIT to all PKI processes -- PKI TIMEMOUT exceeded
-			for (std::list<pid_t>::const_iterator pidi = nick_pids.begin();
-				pidi != nick_pids.end(); pidi++)
+			for (std::list<pid_t>::const_iterator pidi = nick_pids.begin(); pidi != nick_pids.end(); pidi++)
 			{
 				if (nick_ncnt[nick_nick[*pidi]] > PKI_TIMEOUT)
 					if (kill(*pidi, SIGQUIT) < 0)
@@ -2178,8 +2126,7 @@ void run_irc()
 			}
 			
 			// send SIGQUIT to all RNK processes -- RNK TIMEMOUT exceeded
-			for (std::list<pid_t>::const_iterator pidi = rnk_pids.begin();
-				pidi != rnk_pids.end(); pidi++)
+			for (std::list<pid_t>::const_iterator pidi = rnk_pids.begin(); pidi != rnk_pids.end(); pidi++)
 			{
 				if (nick_rnkcnt[rnk_nick[*pidi]] > RNK_TIMEOUT)
 					if (kill(*pidi, SIGQUIT) < 0)
@@ -2187,8 +2134,7 @@ void run_irc()
 			}
 			
 			// start RNK or PKI processes
-			for (std::map<std::string, std::string>::const_iterator ni = nick_players.begin();
-					ni != nick_players.end(); ni++)
+			for (std::map<std::string, std::string>::const_iterator ni = nick_players.begin(); ni != nick_players.end(); ni++)
 			{
 				std::string nick = ni->first, host = ni->second;
 				
@@ -2257,8 +2203,7 @@ void run_irc()
 								perror("run_irc [RNK/child] (close)");
 							
 							// iterate RNK list
-							for (std::vector<std::string>::const_iterator ri = rnk_idlist.begin();
-								ri != rnk_idlist.end(); ri++)
+							for (std::vector<std::string>::const_iterator ri = rnk_idlist.begin(); ri != rnk_idlist.end(); ri++)
 							{
 								// create TCP/IP connection
 								int rhd = ConnectToHost(host.c_str(), nick_p7774[nick]);
@@ -2303,9 +2248,7 @@ void run_irc()
 				}
 				
 				// PKI (obtain and verify public keys of other players)
-				if ((nick_key.find(nick) == nick_key.end()) && 
-					(std::find(nick_ninf.begin(), nick_ninf.end(), nick) 
-					== nick_ninf.end()))
+				if ((nick_key.find(nick) == nick_key.end()) && (std::find(nick_ninf.begin(), nick_ninf.end(), nick) == nick_ninf.end()))
 				{
 					int fd_pipe[2];
 					if (pipe(fd_pipe) < 0)
@@ -2368,7 +2311,7 @@ void run_irc()
 								exit(-3);
 							}
 							
-							// check the NIZK
+							// check the self-signature and NIZK
 							if (!pkey.check())
 							{
 								std::cerr << _("TMCG: invalid public key") << std::endl;
@@ -2398,8 +2341,7 @@ void run_irc()
 						}
 					}
 				}
-				else if (std::find(nick_ninf.begin(), nick_ninf.end(), nick) 
-					!= nick_ninf.end())
+				else if (std::find(nick_ninf.begin(), nick_ninf.end(), nick) != nick_ninf.end())
 				{
 					nick_ncnt[nick] += 1;
 				}
@@ -2408,12 +2350,10 @@ void run_irc()
 	}
     // check whether the IRC connection still exists
     if (!irc->good())
-        std::cerr << _("IRC ERROR: connection with server collapsed") <<
-            std::endl;
+        std::cerr << _("IRC ERROR: connection with server collapsed") << std::endl;
 
     // free the previously allocated memory (read buffers)
-    for (std::map<int, char*>::const_iterator rbi = readbuf.begin(); 
-        rbi != readbuf.end(); rbi++)
+    for (std::map<int, char*>::const_iterator rbi = readbuf.begin(); rbi != readbuf.end(); rbi++)
     {
         delete [] rbi->second;
     }
@@ -2422,25 +2362,22 @@ void run_irc()
 void cleanup()
 {
     // send SIGQUIT to all child processes and wait for them
-    for (std::map<pid_t, std::string>::const_iterator pidi = 
-        games_pid2tnr.begin(); pidi != games_pid2tnr.end(); pidi++)
+    for (std::map<pid_t, std::string>::const_iterator pidi = games_pid2tnr.begin(); pidi != games_pid2tnr.end(); pidi++)
     {
         if (kill(pidi->first, SIGQUIT) < 0)
             perror("cleanup (kill)");
         waitpid(pidi->first, NULL, 0);
     }
     games_pid2tnr.clear(), games_tnr2pid.clear();
-    for (std::list<pid_t>::const_iterator pidi = nick_pids.begin();
-        pidi != nick_pids.end(); pidi++)
+    for (std::list<pid_t>::const_iterator pidi = nick_pids.begin(); pidi != nick_pids.end(); pidi++)
     {
         if (kill(*pidi, SIGQUIT) < 0)
             perror("cleanup (kill)");
         waitpid(*pidi, NULL, 0);
     }
-    nick_pids.clear(), nick_nick.clear(), nick_host.clear(),
-        nick_ninf.clear(), nick_ncnt.clear(), nick_players.clear();
-    for (std::list<pid_t>::const_iterator pidi = rnkrpl_pid.begin();
-        pidi != rnkrpl_pid.end(); pidi++)
+    nick_pids.clear(), nick_nick.clear(), nick_host.clear();
+    nick_ninf.clear(), nick_ncnt.clear(), nick_players.clear();
+    for (std::list<pid_t>::const_iterator pidi = rnkrpl_pid.begin(); pidi != rnkrpl_pid.end(); pidi++)
     {
         if (kill(*pidi, SIGQUIT) < 0)
             perror("cleanup (kill)");
@@ -2489,11 +2426,7 @@ int main(int argc, char* argv[], char* envp[])
 {
     char *home = NULL;
     std::string homedir = "";
-    std::cout << PACKAGE_STRING <<
-        ", (c) 2002--2009  Heiko Stamer <stamer@gaos.org>, License: GPLv2" << 
-        std::endl <<
-        " $Id: SecureSkat.cc,v 1.59 2009/06/11 08:27:45 stamer Exp $ " << 
-        std::endl;
+    std::cout << PACKAGE_STRING << ", (c) 2017  Heiko Stamer <HeikoStamer@gmx.net>, License: GPLv2" << std::endl;
 	
 #ifdef ENABLE_NLS
     // set the locales
@@ -2506,8 +2439,7 @@ int main(int argc, char* argv[], char* envp[])
     // enable the native language support
     bindtextdomain(PACKAGE, LOCALEDIR);
     textdomain(PACKAGE);
-    std::cout << "++ " << _("Internationalization support") << ": " << 
-        LOCALEDIR << std::endl;
+    std::cout << "++ " << _("Internationalization support") << ": " << LOCALEDIR << std::endl;
 #endif
 	
     // evaluate the environment variable HOME
@@ -2516,8 +2448,7 @@ int main(int argc, char* argv[], char* envp[])
         homedir = home, homedir += "/.SecureSkat/";
     else
         homedir = "~/.SecureSkat/";
-    std::cout << "++ " << _("PKI/RNK database directory") << ": " << 
-        homedir << std::endl;
+    std::cout << "++ " << _("PKI/RNK database directory") << ": " << homedir << std::endl;
     
     // check existance and permissions of the home directory
     struct stat stat_buffer;
@@ -2528,16 +2459,14 @@ int main(int argc, char* argv[], char* envp[])
             // create directory, if it doesn't exist
             if (mkdir(homedir.c_str(), S_IRUSR | S_IWUSR | S_IXUSR))
             {
-                std::cerr << _("Can't create directory!") << " (" << 
-                    strerror(errno) << ")" << std::endl;
+                std::cerr << _("Can't create directory!") << " (" << strerror(errno) << ")" << std::endl;
                 return EXIT_FAILURE;
             }
         }
         else
         {
             // print error message
-            std::cerr << _("Can't get the status of the directory!") << 
-                " (" << strerror(errno) << ")" << std::endl;
+            std::cerr << _("Can't get the status of the directory!") << " (" << strerror(errno) << ")" << std::endl;
             return EXIT_FAILURE;
         }
     }
@@ -2553,8 +2482,7 @@ int main(int argc, char* argv[], char* envp[])
             std::cerr << _("Wrong owner of the directory!") << std::endl;
             return EXIT_FAILURE;
         }
-        if ((stat_buffer.st_mode & (S_IRUSR | S_IWUSR | S_IXUSR)) !=
-            (S_IRUSR | S_IWUSR | S_IXUSR))
+        if ((stat_buffer.st_mode & (S_IRUSR | S_IWUSR | S_IXUSR)) != (S_IRUSR | S_IWUSR | S_IXUSR))
         {
             std::cerr << _("Missing permissions for directory!") << std::endl;
             return EXIT_FAILURE;
@@ -2562,8 +2490,7 @@ int main(int argc, char* argv[], char* envp[])
     }
 	
     // process the command line arguments
-    if (((argc == 4) && isdigit(argv[2][0])) ||
-        ((argc == 3) && isdigit(argv[2][0])) ||	(argc == 2))
+    if (((argc == 4) && isdigit(argv[2][0])) || ((argc == 3) && isdigit(argv[2][0])) ||	(argc == 2))
     {
         struct termios old_term;
         int irc_port = 6667;
@@ -2571,7 +2498,7 @@ int main(int argc, char* argv[], char* envp[])
         // set the default values
         game_ctl = "";
         game_env = NULL;
-		
+
         // evaluate the provided command switches to override the defaults
         switch (argc)
         {
@@ -2588,12 +2515,13 @@ int main(int argc, char* argv[], char* envp[])
             std::cerr << _("Initialization of LibTMCG failed!") << std::endl;
             return EXIT_FAILURE;
         }
+        // display version of LibTMCG
+        std::cout << "++ " << _("Initialization of LibTMCG version") << " " << version_libTMCG() << std::endl;
 
         // key management		
         get_secret_key(homedir + "SecureSkat.skr", sec, public_prefix);
         pub = TMCG_PublicKey(sec); // extract the public part of the secret key
-        std::cout << _("Your key fingerprint") << ": " << 
-            pub.fingerprint() << std::endl;
+        std::cout << _("Your key fingerprint") << ": " << pub.fingerprint() << std::endl;
         get_public_keys(homedir + "SecureSkat.pkr", nick_key); // load pub keys
 		
         create_pki(pki7771_port, pki7771_handle);
@@ -2617,9 +2545,7 @@ int main(int argc, char* argv[], char* envp[])
         signal(SIGUSR1, sig_handler_usr1);
 
         init_irc(irc, pub.keyid(5));
-        std::cout << _("Usage") << ": " <<
-            _("type /help for the command list or read the file README") <<
-            std::endl;
+        std::cout << _("Usage") << ": " << _("type /help for the command list or read the file README") << std::endl;
 #ifndef NOHUP
         init_term(old_term);
 #endif
@@ -2649,8 +2575,6 @@ int main(int argc, char* argv[], char* envp[])
     }
 
     // print a short usage message and exit with failure
-    std::cout << _("Usage") << ": " << argv[0] <<
-        " IRC_SERVER<string> [ IRC_PORT<int> [ CTRL_PROGRAM<string> ] ]" << 
-        std::endl;
+    std::cout << _("Usage") << ": " << argv[0] << " IRC_SERVER<string> [ IRC_PORT<int> [ CTRL_PROGRAM<string> ] ]" << std::endl;
     return EXIT_FAILURE;
 }
