@@ -1,7 +1,8 @@
 /*******************************************************************************
    This file is part of SecureSkat.
 
- Copyright (C) 2002, 2003, 2004, 2006, 2007, 2009 Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2002, 2003, 2004, 2006, 2007, 2009
+                                       2017 Heiko Stamer <HeikoStamer@gmx.net>
 
    SecureSkat is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,79 +24,94 @@
 int BindEmptyPort
 	(int start_port)
 {
-	int current_port = start_port;
-	while (1)
+	const int max_ports = 100;
+	int current_port = start_port + (mpz_wrandom_ui() % (max_ports / 2));
+	while (current_port < (start_port + max_ports))
 	{
-		int socket_handle;
+		int sockfd;
 		long socket_option = 1;
-		struct sockaddr_in sin;
-		sin.sin_port = htons(current_port), sin.sin_family = AF_INET;
+		struct sockaddr_in sin = { 0 };
+		sin.sin_port = htons(current_port);
+		sin.sin_family = AF_INET;
 		sin.sin_addr.s_addr = htonl(INADDR_ANY);
-		if ((socket_handle = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+		if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
 		{
 			perror("SecureSkat_misc::BindEmptyPort (socket)");
 			return -1;
 		}
-		if (setsockopt(socket_handle, SOL_SOCKET, SO_REUSEADDR, &socket_option,
-			sizeof(socket_option)) < 0)
+		if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &socket_option, sizeof(socket_option)) < 0)
 		{
 			perror("SecureSkat_misc::BindEmptyPort (setsockopt)");
+			if (close(sockfd) < 0)
+				perror("SecureSkat_misc::BindEmptyPort (close)");
 			return -2;
 		}
-		if (bind(socket_handle, (struct sockaddr*)&sin, sizeof(sin)) < 0)
+		if (bind(sockfd, (struct sockaddr*)&sin, sizeof(sin)) < 0)
 		{
 			current_port++;
 		}
 		else
 		{
-			if (close(socket_handle) < 0)
+			if (close(sockfd) < 0)
 			{
 				perror("SecureSkat_misc::BindEmptyPort (close)");
 				return -3;
 			}
 			break;
 		}
+		if (close(sockfd) < 0)
+		{
+			perror("SecureSkat_misc::BindEmptyPort (close)");
+			return -3;
+		}
 	}
+	if (current_port == (start_port + max_ports))
+		return -4;
 	return current_port;
 }
 
 int ListenToPort
 	(int port)
 {
-	int handle;
+	int sockfd;
 	long socket_option = 1;
-	struct sockaddr_in sin;
-	sin.sin_addr.s_addr = htonl(INADDR_ANY);
+	struct sockaddr_in sin = { 0 };
+	sin.sin_port = htons(port);
 	sin.sin_family = AF_INET;
-	if ((handle = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+	sin.sin_addr.s_addr = htonl(INADDR_ANY);
+	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
 	{
 		perror("SecureSkat_misc::ListenToPort (socket)");
 		return -1;
 	}
-	if (setsockopt(handle, SOL_SOCKET, SO_REUSEADDR, &socket_option,
-		sizeof(socket_option)) < 0)
+	if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &socket_option, sizeof(socket_option)) < 0)
 	{
 		perror("SecureSkat_misc::ListenToPort (setsockopt)");
+		if (close(sockfd) < 0)
+			perror("SecureSkat_misc::ListenToPort (close)");
 		return -2;
 	}
-	sin.sin_port = htons(port);
-	if (bind(handle, (struct sockaddr*)&sin, sizeof(sin)) < 0)
+	if (bind(sockfd, (struct sockaddr*)&sin, sizeof(sin)) < 0)
 	{
 		perror("SecureSkat_misc::ListenToPort (bind)");
+		if (close(sockfd) < 0)
+			perror("SecureSkat_misc::ListenToPort (close)");
 		return -3;
 	}
-	if (listen(handle, SOMAXCONN) < 0)
+	if (listen(sockfd, SOMAXCONN) < 0)
 	{
 		perror("SecureSkat_misc::ListenToPort (listen)");
+		if (close(sockfd) < 0)
+			perror("SecureSkat_misc::ListenToPort (close)");
 		return -4;
 	}
-	return handle;
+	return sockfd;
 }
 
 int CloseHandle
-	(int handle)
+	(int sockfd)
 {
-	if (close(handle) < 0)
+	if (close(sockfd) < 0)
 	{
 		perror("SecureSkat_misc::CloseHandle (close)");
 		return -1;
@@ -106,10 +122,11 @@ int CloseHandle
 int ConnectToHost
 	(const char *host, int port)
 {
-	int handle;
+	int sockfd;
 	struct hostent *hostinf;
-	struct sockaddr_in sin;
-	sin.sin_port = htons(port), sin.sin_family = AF_INET;
+	struct sockaddr_in sin = { 0 };
+	sin.sin_port = htons(port);
+	sin.sin_family = AF_INET;
 	if ((hostinf = gethostbyname(host)) != NULL)
 	{
 		memcpy((char*)&sin.sin_addr, hostinf->h_addr, hostinf->h_length);
@@ -119,31 +136,31 @@ int ConnectToHost
 		perror("SecureSkat_misc::ConnectToHost (gethostbyname)");
 		return -1;
 	}
-	if ((handle = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
 	{
 		perror("SecureSkat_misc::ConnectToHost (socket)");
 		return -2;
 	}
-	if ((connect(handle, (struct sockaddr*)&sin, sizeof(sin))) < 0)
+	if ((connect(sockfd, (struct sockaddr*)&sin, sizeof(sin))) < 0)
 	{
 		perror("SecureSkat_misc::ConnectToHost (connect)");
+		if (close(sockfd) < 0)
+			perror("SecureSkat_misc::ConnectToHost (close)");
 		return -3;
 	}
-	return handle;
+	return sockfd;
 }
 
 char *stripwhite(char *str)
 {
-	register char *s, *t;
-	
-	for (s = str; ((*s == ' ') || (*s == '\t')); s++)
-		;
+	register char *s = str, *t = NULL;
 	
+	while ((*s == ' ') || (*s == '\t'))
+		s++;
 	if (*s == 0)
 		return s;
-	
 	t = s + strlen(s) - 1;
-	while (t > s && ((*t == ' ') || (*t == '\t')))
+	while ((t > s) && ((*t == ' ') || (*t == '\t')))
 		t--;
 	*++t = '\0';
 	
@@ -151,7 +168,7 @@ char *stripwhite(char *str)
 }
 
 clock_t start_time, stop_time;
-char time_buffer[100];
+char time_buffer[128];
 
 void start_clock
 	(void)
